# Python script for getting images of wavefields
import sys
import numpy
import matplotlib.pyplot as matplot
import matplotlib.cm
from matplotlib.colors import LinearSegmentedColormap

case = float(sys.argv[1])
# 0 to make marmFlt.pdf
# 0.5 to make marmFlt.pdf in black & white
# 1 to make marmDbl.pdf
# 1.5 to make marmDbl.pdf BW
# 2 to make layerFlt.pdf
# 2.5 to make layerFlt.pdf BW
# 3 to make layerDbl.pdf
# 3.5 to make layerDbl.pdf BW
# 4 to make avgDiff.pdf 
# 4.5 to make avgDiff.pdf BW

datanames = []
imagename = 'dummyname'

# make plots in series of four (no compression, 4/3x , 2x , and 4x compression)
if(case == 0 or case == 0.5):
	nx = 384
	nz = 122
	datanames = ['Inter/marmhard500prec32f.txt', 
			 'Inter/marmhard500prec24f.txt', 
			 'Inter/marmhard500prec16f.txt', 
			 'Inter/marmhard500prec8f.txt']
        if(case == 0):
	  imagename = 'Fig/marmFlt.pdf'
        else:
          imagename = 'FigBW/marmFlt.pdf'
elif(case == 1 or case == 1.5):
	nx = 384
	nz = 122
	datanames = ['Inter/marmhard500prec64.txt', 
			 'Inter/marmhard500prec48.txt', 
			 'Inter/marmhard500prec32.txt', 
			 'Inter/marmhard500prec16.txt']
        if(case == 1):
	  imagename = 'Fig/marmDbl.pdf'
        else:
          imagename = 'FigBW/marmDbl.pdf'
elif(case == 2 or case == 2.5):
	nx = 1000
	nz = 1000
	datanames = ['Inter/twoLayer800prec32f.txt',
				'Inter/twoLayer800prec24f.txt',
				'Inter/twoLayer800prec16f.txt',
				'Inter/twoLayer800prec8f.txt']
        if(case == 2):
	  imagename = 'Fig/layerFlt.pdf'
        else:
          imagename = 'FigBW/layerFlt.pdf'	
elif(case == 3 or case == 3.5):
	nx = 1000
	nz = 1000
	datanames = ['Inter/twoLayer800prec64.txt',
				'Inter/twoLayer800prec48.txt',
				'Inter/twoLayer800prec32.txt',
				'Inter/twoLayer800prec16.txt']
        if(case == 3):
	  imagename = 'Fig/layerDbl.pdf'
        else:
          imagename = 'FigBW/layerDbl.pdf'
elif(case == 4 or case == 4.5):
	datanames = ['Inter/layerDiffDbl.txt', 'Inter/layerDiffFlt.txt']
	if(case == 4):
	  imagename = 'Fig/avgDiff.pdf'
	else:
	  imagename = 'FigBW/avgDiff.pdf'

if(case in [0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5]):
	if(case in [0, 1, 2, 3]):
		# Design a red white blue colorbar
		cut = 0.1
		colorList = { 'red': ((0,0,0),(.5-cut,0,0),(0.5,1,1),(.5+cut,1,1),(1,0.4,1)),
			  'green': ((0,0,0),(.5-cut,0,0),(0.5,1,1),(.5+cut,0,0),(1,0,0)),
			  'blue': ((0,0,0),(.5-cut,1,1),(0.5,1,1),(0.5+cut,0,0),(1,0,0))
			}
		rwb = LinearSegmentedColormap('RedWhiteBlue',colorList)
	 
	for i in range(0,4):
		a = numpy.fromfile(datanames[i],'d',-1," ").reshape(nx,nz)
		numpy.float32(a)
		valmax = a.flat[abs(a).argmax()]
		a *= 0.5/valmax
		a += 0.5
		anorm = matplotlib.colors.Normalize(vmin=0, vmax=1)
		if(case <= 1.5): # horizontally arranged
			matplotlib.pyplot.subplot(4,1,i+1)
		elif(case >= 2): # vertically arranged
			matplotlib.pyplot.subplot(1,4,i+1)
                if(case == 0 or case == 1 or case == 2 or case == 3):
			aimg = matplotlib.pyplot.imshow(a.T, norm=anorm, interpolation='nearest', cmap=rwb)
                else:
                	aimg = matplotlib.pyplot.imshow(a.T, norm = anorm, interpolation='nearest', cmap = 'gray')
		aimg.axes.get_xaxis().set_visible(False)
		aimg.axes.get_yaxis().set_visible(False)
	matplotlib.pyplot.savefig(imagename,format='pdf')

elif(case == 4 or case == 4.5):
	# set up time steps
	numsteps = 8;
	timesteps = numpy.array( [100., 200., 300., 400., 500., 600., 700., 800.] )
	timesteps *= 0.002

	# read arrays from files
	dblAvgRelDiff = numpy.loadtxt(datanames[0])
	dbl48AvgRelDiff = dblAvgRelDiff[numsteps:2*numsteps]
	dbl32AvgRelDiff = dblAvgRelDiff[2*numsteps:3*numsteps]
	dbl16AvgRelDiff = dblAvgRelDiff[3*numsteps:4*numsteps]
	fltAvgRelDiff = numpy.loadtxt(datanames[1])
	flt24AvgRelDiff = fltAvgRelDiff[numsteps:2*numsteps]
	flt16AvgRelDiff = fltAvgRelDiff[2*numsteps:3*numsteps]
	flt8AvgRelDiff = fltAvgRelDiff[3*numsteps:4*numsteps]

	# do the plots in color, then black and white
	matplot.hold(True)
	matplot.ylim(-18,1)
	if(case == 4):
		matplot.plot(timesteps,numpy.log10(dbl16AvgRelDiff),'--',marker='o',color='b',label="16 bit double")
		matplot.plot(timesteps,numpy.log10(dbl32AvgRelDiff),'--',marker='*',color='g',label="32 bit double")
		matplot.plot(timesteps,numpy.log10(dbl48AvgRelDiff),'--',marker='D',color='r',label="48 bit double")
		matplot.plot(timesteps,numpy.log10(flt8AvgRelDiff),'-',marker='o',color='b',label="8 bit float")
		matplot.plot(timesteps,numpy.log10(flt16AvgRelDiff),'-',marker='*',color='g',label="16 bit float")
		matplot.plot(timesteps,numpy.log10(flt24AvgRelDiff),'-',marker='D',color='r',label="24 bit float")
	if(case == 4.5):
		matplot.plot(timesteps,numpy.log10(dbl16AvgRelDiff),'--',marker='o',color='k',label="16 bit double")
		matplot.plot(timesteps,numpy.log10(dbl32AvgRelDiff),'--',marker='*',color='k',label="32 bit double")
		matplot.plot(timesteps,numpy.log10(dbl48AvgRelDiff),'--',marker='D',color='k',label="48 bit double")
		matplot.plot(timesteps,numpy.log10(flt8AvgRelDiff),'-',marker='o',color='k',label="8 bit float")
		matplot.plot(timesteps,numpy.log10(flt16AvgRelDiff),'-',marker='*',color='k',label="16 bit float")
		matplot.plot(timesteps,numpy.log10(flt24AvgRelDiff),'-',marker='D',color='k',label="24 bit float")
	matplot.xlabel('Time (s)',fontsize=18)
	matplot.ylabel('Log of average relative difference',fontsize=18)
	matplot.legend(loc='lower right')
	matplot.savefig(imagename,format='pdf')
	matplot.close()
