% read a velocity model from file, make sure it's read properly with a plot
fileID = fopen('Data/marmhard.dat','r');
formatSpec = '%f';
vel = fscanf(fileID,formatSpec);
maxvel = max(vel);
length = 9192+24;
width = 2904+24;
dxz = 24;
nx = length/dxz;
nz = width/dxz;
vel = reshape(vel,nz,nx);
% set up time discretization, source
f = 10; % peak source frequency
dt = min(0.5*dxz/maxvel, 1/(2.1*f));
nt = 401; % number of time steps
writeStep = 400; % number of time steps until compression
sx = nx/2;
sz = nz/2;
t0 = 10; % time offset
param = (pi*f*(t0-[1:nt]')*dt).^2;
source = (1-2*param).*exp(-param);

% set up wavelet 
for basisType = 1:2
  color = 1; % color = 1 or black and white = 0

  switch basisType
    case 1
      wave_name = 'bior4.4';
      comp_level = 2; % decomposition level
      thr = 10;
    case 2
      comp_level = 2;% used for individual
      wave_name='curvelet'; % not actually used, just to have some filler
      thr = 15;
  end
    opt = 'gbl'; % global threshold
    sorh = 'h'; % hard threshold
    keepapp = 1; % approx coeff cannot be thresholded

    % setup of initial wave field 
    oldfieldC = zeros(nz,nx);
    oldfield = zeros(nz,nx);
    wavefieldC = zeros(nz,nx);
    wavefieldC(sz,sx) = source(1);
    wavefield = zeros(nz,nx);
    wavefield(sz,sx) = source(1);
    newfield = zeros(nz,nx);
    newfieldC = zeros(nz,nx);




    % to measure errors
    DiffFroNorm = [];
    CompFactor = [];

    % run time steps
    for it =1:nt


      if(it == 1)
        oldfieldC = oldfieldC; % decompression already done
        wavefieldC = wavefieldC; % decompression already done
      else
        oldfieldC = wavefieldC;
        wavefieldC = newfieldC;
        oldfield = wavefield;
        wavefield = newfield;
      end

      % calculate next time step
      newfieldC = [zeros(1,nx-2); ((vel(2:nz-1,2:nx-1)*dt/dxz).^2).*(wavefieldC(3:nz,2:nx-1)+wavefieldC(1:nz-2,2:nx-1)-4*wavefieldC(2:nz-1,2:nx-1)+wavefieldC(2:nz-1,3:nx)+wavefieldC(2:nz-1,1:nx-2))+2*wavefieldC(2:nz-1,2:nx-1)-oldfieldC(2:nz-1,2:nx-1); zeros(1,nx-2)];
      newfieldC = [zeros(nz,1) newfieldC zeros(nz,1)];
      newfieldC(sz,sx) = newfieldC(sz,sx) + source(it);
      newfield = [zeros(1,nx-2); ((vel(2:nz-1,2:nx-1)*dt/dxz).^2).*(wavefield(3:nz,2:nx-1)+wavefield(1:nz-2,2:nx-1)-4*wavefield(2:nz-1,2:nx-1)+wavefield(2:nz-1,3:nx)+wavefield(2:nz-1,1:nx-2))+2*wavefield(2:nz-1,2:nx-1)-oldfield(2:nz-1,2:nx-1); zeros(1,nx-2)];
      newfield = [zeros(nz,1) newfield zeros(nz,1)];
      newfield(sz,sx) = newfield(sz,sx) + source(it);

      % error calculations
      DiffFroNorm = [DiffFroNorm ; (norm(newfieldC-newfield,'fro'))/(norm(newfield,'fro'))];

      % compress newfield, wavefield
      if(it == writeStep )
        [wavefieldC,c2w,l2w,l2normRecw,compScorew] = compress(wavefieldC,wave_name, comp_level, opt, thr, sorh, keepapp,basisType);
        [newfieldC, c2n,l2n,l2normRecn,compScoren] = compress(newfieldC, wave_name, comp_level, opt, thr, sorh, keepapp,basisType);
        CompFactor = [CompFactor; compScoren];

        maxvalue = max(max(max(wavefield)),max(max(wavefieldC)));
        minvalue = min(min(min(wavefield)),min(min(wavefieldC)));
        limit = max(abs(maxvalue),abs(minvalue));

        % color RWB image of wave fields
         if(basisType == 1)
            wavepic = figure;
            subplot(3,1,1);
            imagesc(wavefieldC);
            waveletWavefield = wavefieldC; % so it can be used for black and white
            caxis([-limit,limit]);
            colormap('redbluecmap');
            axis equal;
            hold on;
            axis off;
            subplot(3,1,2);
            imagesc(wavefield);
            uncompressedWavefield = wavefield; % so it can be used for black and white
            caxis([-limit,limit]);
            colormap('redbluecmap');
            axis equal;
            hold on;
            axis off;
        elseif(basisType == 2)
          subplot(3,1,3);
          imagesc(wavefieldC);
          curveletWavefield = wavefieldC; % so it can be used for black and white
          caxis([-limit,limit]);
          colormap('redbluecmap');
          axis equal;
          hold on;
          axis off;
          print(wavepic,'-dpdf', 'Fig/individual.pdf');
         end
      end
    %pause%(0.01);
  end
end

% black and white image of wave fields
wavepic = figure;
subplot(3,1,1);
imagesc(waveletWavefield);
caxis([-limit,limit]);
colormap('gray');
axis equal;
hold on;
axis off;
subplot(3,1,2);
imagesc(uncompressedWavefield);
caxis([-limit,limit]);
colormap('gray');
axis equal;
hold on;
axis off;
subplot(3,1,3);
imagesc(curveletWavefield);
caxis([-limit,limit]);
colormap('gray');
axis equal;
hold on;
axis off;
print(wavepic,'-dpdf', 'FigBW/individual.pdf');
