% read a velocity model from file, make sure it's read properly with a plot
fileID = fopen('Data/marmhard.dat','r');
formatSpec = '%f';
vel = fscanf(fileID,formatSpec);
maxvel = max(vel);
length = 9192+24;
width = 2904+24;
dxz = 24;
nx = length/dxz;
nz = width/dxz;
vel = reshape(vel,nz,nx);
%imagesc(vel);
% set up time discretization, source
f = 10; % peak source frequency
dt = min(0.5*dxz/maxvel, 1/(2.1*f));
nt = 401; % number of time steps
compFrq = 50; % number of time steps between compressions
sx = nx/2;
sz = nz/2;
t0 = 10; % time offset
param = (pi*f*(t0-[1:nt]')*dt).^2;
source = (1-2*param).*exp(-param);

%imagesc(vel);

% set up wavelet 
 % color = 1 or black and white = 0

switch basisType
  case 1
    wave_name = 'bior4.4';
    comp_level = 1; % decomposition level
    thr = 10;
    errFigName = 'Fig/errorWave.pdf';
    BWerrFigName = 'FigBW/errorWave.pdf';
    ratioFigName = 'Fig/ratioWave.pdf';
    BWratioFigName = 'FigBW/ratioWave.pdf';
  case 2
    comp_level = 2;% used for individual
    wave_name='curvelet'; % not actually used, just to have some filler
    thr = 50; % any coeff. at least 1/50*the max coeff. will be kept
    errFigName = 'Fig/errorCurve.pdf';
    BWerrFigName = 'FigBW/errorCurve.pdf';
    ratioFigName = 'Fig/ratioCurve.pdf';
    BWratioFigName = 'FigBW/ratioCurve.pdf';
end
opt = 'gbl'; % global threshol
sorh = 'h'; % hard threshold
keepapp = 1; % approx coeff cannot be thresholded

% setup of initial wave field 
oldfieldC = zeros(nz,nx);
oldfield = zeros(nz,nx);
wavefieldC = zeros(nz,nx);
wavefieldC(sz,sx) = source(1);
wavefield = zeros(nz,nx);
wavefield(sz,sx) = source(1);
newfield = zeros(nz,nx);
newfieldC = zeros(nz,nx);




% to measure errors
DiffFroNorm = [];
CompFactor = [];

% run time steps
for it =1:nt

  % decompress previous time step field 
  if(mod(it,compFrq) == 1 && it > 1)
    oldfieldC = wavefieldC; % decompression already done
    wavefieldC = newfieldC; % decompression already done
    oldfield = wavefield;
    wavefield = newfield;
  elseif(it == 1)
    oldfieldC = oldfieldC; % decompression already done
    wavefieldC = wavefieldC; % decompression already done
  else
    oldfieldC = wavefieldC;
    wavefieldC = newfieldC;
    oldfield = wavefield;
    wavefield = newfield;
  end

  % calculate next time step
  newfieldC = [zeros(1,nx-2); ((vel(2:nz-1,2:nx-1)*dt/dxz).^2).*(wavefieldC(3:nz,2:nx-1)+wavefieldC(1:nz-2,2:nx-1)-4*wavefieldC(2:nz-1,2:nx-1)+wavefieldC(2:nz-1,3:nx)+wavefieldC(2:nz-1,1:nx-2))+2*wavefieldC(2:nz-1,2:nx-1)-oldfieldC(2:nz-1,2:nx-1); zeros(1,nx-2)];
  newfieldC = [zeros(nz,1) newfieldC zeros(nz,1)];
  newfieldC(sz,sx) = newfieldC(sz,sx) + source(it);
  newfield = [zeros(1,nx-2); ((vel(2:nz-1,2:nx-1)*dt/dxz).^2).*(wavefield(3:nz,2:nx-1)+wavefield(1:nz-2,2:nx-1)-4*wavefield(2:nz-1,2:nx-1)+wavefield(2:nz-1,3:nx)+wavefield(2:nz-1,1:nx-2))+2*wavefield(2:nz-1,2:nx-1)-oldfield(2:nz-1,2:nx-1); zeros(1,nx-2)];
  newfield = [zeros(nz,1) newfield zeros(nz,1)];
  newfield(sz,sx) = newfield(sz,sx) + source(it);

  % error calculations
  DiffFroNorm = [DiffFroNorm ; (norm(newfieldC-newfield,'fro'))/(norm(newfield,'fro'))];
  
  % compress newfield, wavefield
    if(mod(it, compFrq) == 0 )%&& it > 50)\
      [wavefieldC,c2w,l2w,l2normRecw,compScorew] = compress(wavefieldC,wave_name, comp_level, opt, thr, sorh, keepapp,basisType);
      [newfieldC, c2n,l2n,l2normRecn,compScoren] = compress(newfieldC, wave_name, comp_level, opt, thr, sorh, keepapp,basisType);
      CompFactor = [CompFactor; compScoren];
    end
end

% plot relative errors
err = figure;
scatter([1:nt]',DiffFroNorm,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
xlabel('Time steps','FontSize',12,'FontWeight','bold');
ylabel('Relative error','FontSize',12,'FontWeight','bold');
if(basisType == 1)
   title('Level 1 CDF 9/7 Wavelet Compression','FontSize',14,'FontWeight','bold'); 
else
  title('8 Angle, Level 2 Curvelet Compression','FontSize',14,'FontWeight','bold');
end
print(err,'-dpdf', errFigName);
print(err,'-dpdf', BWerrFigName);
% plot compression ratios
ratio = figure;
scatter([compFrq:compFrq:nt]',CompFactor,'o','MarkerFaceColor','k','MarkerEdgeColor','k');
xlabel('Time steps','FontSize',12,'FontWeight','bold');
ylabel('No. nonzeros / n_x*n_z','FontSize',12,'FontWeight','bold');
if(basisType == 1)
   title('Level 1 CDF 9/7 Wavelet Compression','FontSize',14,'FontWeight','bold'); 
else
  title('8 Angle, Level 2 Curvelet Compression','FontSize',14,'FontWeight','bold');
end
print(ratio,'-dpdf', ratioFigName);
print(ratio,'-dpdf', BWratioFigName);
