#include <climits>
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <cmath>
#include <fstream>
#include <string>
#include <vector>
#include <typeinfo>
#include <sys/time.h>
#include "fpzip.h"

#ifndef TYPE
  #error "must define TYPE as float or double in Makefile"
#endif

#define string_(s) #s
#define string(s) string_(s)

static double
now()
{
  struct timeval tv;
  gettimeofday(&tv, 0);
  return tv.tv_sec + 1e-6 * tv.tv_usec;
}

int main(int argc, char* argv[]){
  double factor [4] = { 1, 0.75, 0.5, 0.25 };
  typedef TYPE dummyvar;
  std::vector<TYPE> avgRelDiff;
  std::string filename800 [4] = {"o","o","o","o"};
  if(typeid(dummyvar) == typeid(double)){
    filename800[0] = "Inter/twoLayer800prec64.txt";
    filename800[1] = "Inter/twoLayer800prec48.txt"; 
    filename800[2] = "Inter/twoLayer800prec32.txt"; 
    filename800[3] = "Inter/twoLayer800prec16.txt";
  } else if(typeid(dummyvar) == typeid(float)){
    filename800[0] = "Inter/twoLayer800prec32f.txt";
    filename800[1] = "Inter/twoLayer800prec24f.txt"; 
    filename800[2] = "Inter/twoLayer800prec16f.txt"; 
    filename800[3] = "Inter/twoLayer800prec8f.txt"; 
  }

 for(int compi=0; compi<4; compi++){
  int length = 16000; // in meters
  int width = 16000; // in meters
  int dxz = 16; // spacing in meters
  int dim[4] = {length/dxz, width/dxz, 1, 1};

  // Set up the domain and velocity model 
  int nx = dim[0]; // number of increments
  int nz = dim[1];
  double dt = 0.002; // time spacing
  int nt = 805;  // just for testing compression 

  double* vel = new double[nx*nz];
  for(int i=0; i<nx; ++i){
    for(int j=0; j<nx/4; ++j){
      vel[i*nx+j] = 3000;
    }
    for(int j=nx/4; j<nz; ++j){
      vel[i*nx+j] = 4000;
    }
  }
  // set up precision
  int dp = (sizeof(TYPE) == sizeof(double));
  int size = nx*nz;
  //unsigned int inbytes = size*sizeof(TYPE);
  int* prec = new int[1];
  double compFactor = factor[compi]; // use 1 for lossless compression
  for(int i=0; i<dim[3]; ++i){
    prec[i] = CHAR_BIT*sizeof(TYPE)*compFactor;
    std::cout << "prec is " << prec[i] << std::endl; 
  }
  // Set up Ricker source wavelet and single source position 
  double source[nt]; 
  int sx = 500*nx + 500; // source index position (1000 m down 1000 m left-right)
  int t0 = 10; // time offset
  double pi = 3.1415926535;
  double f = 10; // peak frequency in Hz
  for(int i=0; i<nt; ++i){ 
    double param = std::pow(pi*f*double(t0-i)*dt,2); 
    source[i] = (1-2*param)*std::exp(-param);
  }

  char *outfile = "Inter/outfile.fpz";
  TYPE* oldfieldC =new TYPE[size];
  TYPE* oldfield = new TYPE[size];
  TYPE* wavefieldC = new TYPE[size];
  TYPE* wavefield = new TYPE[size];
  TYPE* newfieldC = new TYPE[size];
  TYPE* newfield = new TYPE[size];
  for(int i=0; i<size; ++i){
    oldfieldC[i] = 0;
    oldfield[i] = 0;
    if(i == sx){ 
      wavefieldC[i] = source[0];
      wavefield[i] = source[0];
    } else{ 
      wavefieldC[i] = 0;
      wavefield[i] = 0;
    }
  }

  TYPE diffFroNorm = 0.0; // frobenious norm of difference in wavefields summed over time
  TYPE FroNorm = std::pow(source[0], 2); // frobenious norm of wavefield
  TYPE avgRelDiffFroNorm = 0.0; // average of  (fro norm of diff. divided by fro norm of wavefield)
  FILE* file = fopen(outfile, "wb");
  if(!file){
    fprintf(stderr, "First open failed \n");
    return EXIT_FAILURE; 
  }
  unsigned int outbytes = fpzip_file_write(file, wavefieldC, prec, dp, dim[0], dim[1], dim[2], dim[3]);
  fclose(file);
  if(!outbytes){
    fprintf(stderr, "compression failed \n");
    return EXIT_FAILURE;
  }
  

  for(int it=1; it<nt; ++it){
    // for each time step, read compressed wavefield 
    file = fopen(outfile, "rb");
    if(!file){
      fprintf(stderr, "open failed \n");
      return EXIT_FAILURE;
    }
    // read and decompress wavefield from file
    unsigned int inbytes = fpzip_file_read(file, wavefieldC, prec, dp, dim[0], dim[1], dim[2], dim[3]);
    if(!inbytes){
      fprintf(stderr, "decompression failed\n");
    }
    fclose(file);

    
    TYPE ThisDiffFroNorm = 0;
    TYPE ThisFroNorm = 0;
    // Calculate next step
    for(int j=0; j<nz; ++j){
      newfieldC[0*nx+j] = 0;
      newfield[0*nx+j] = 0;
      oldfield[0*nx+j] = 0;
      oldfield[0*nx+j] = 0;
    }
    for(int i=1; i<nx-1; ++i){
      newfieldC[i*nx+0] = 0;
      newfield[i*nx+0] = 0;
      oldfieldC[i*nx+0] = 0;
      oldfield[i*nx+0] = 0;
      for(int j=1; j<nz-1; ++j){
        newfieldC[i*nx+j] = std::pow(vel[i*nx+j]*dt/dxz,2)*(wavefieldC[(i-1)*nx+j]+ wavefieldC[i*nx+j-1]-4*wavefieldC[i*nx+j]+wavefieldC[i*nx+j+1] + wavefieldC[(i+1)*nx+j])+2*wavefieldC[i*nx+j]-oldfieldC[i*nx+j];
        newfield[i*nx+j] = std::pow(vel[i*nx+j]*dt/dxz,2)*(wavefield[(i-1)*nx+j]+ wavefield[i*nx+j-1]-4*wavefield[i*nx+j]+wavefield[i*nx+j+1] + wavefield[(i+1)*nx+j])+2*wavefield[i*nx+j]-oldfield[i*nx+j];
        if((i*nx+j) == sx){
          newfieldC[i*nx+j] += source[it];
          newfield[i*nx+j] += source[it];
        }

        ThisDiffFroNorm += std::pow(newfieldC[i*nx+j]-newfield[i*nx+j], 2); // add to difference in frobenious norm
        ThisFroNorm += std::pow(newfield[i*nx+j], 2); // add to frobenious norm
        oldfieldC[i*nx+j] = wavefieldC[i*nx+j]; // set old from time step i-2 to i-1
        oldfield[i*nx+j] = wavefield[i*nx+j];
      } 
      newfieldC[i*nx+nz-1] = 0;
      newfield[i*nx+nz-1] = 0;
      oldfieldC[i*nx+nz-1] = 0;   
      oldfield[i*nx+nz-1] = 0;
    }
    for(int j=0; j<nz; ++j){
      newfieldC[(nx-1)*nx+j] = 0;
      newfield[(nx-1)*nx+j] = 0;
      oldfieldC[(nx-1)*nx+j] = 0;
      oldfield[(nx-1)*nx+j] = 0;
    }

    if(it==800){
        std::ofstream filestream;
        if(it==800) filestream.open((filename800[compi]).c_str());
        if(filestream.good()){
          for(int i=0; i<nx; ++i){
            for(int j=0; j<nz; ++j){
              filestream << newfieldC[i*nz+j] << " ";
            }
            filestream << std::endl;
          }
        } else{
          std::cout << "Input file was no good. \n";
          return 0;
        }
      filestream.close();
    }



    // Output about difference
    avgRelDiffFroNorm += ThisDiffFroNorm/ThisFroNorm;

    // write compressed wavefield to file
    file = fopen(outfile, "wb");
    if(!file){
      fprintf(stderr, "open failed \n");
      return EXIT_FAILURE;
    }
    outbytes = fpzip_file_write(file, newfieldC, prec, dp, dim[0], dim[1], dim[2], dim[3]);
    fclose(file);
    if(!outbytes){
      fprintf(stderr, "compression failed\n");
      return EXIT_FAILURE;
    }
    
    // update wavefield to take next time step
    for(int i=0; i<nx; ++i){
      for(int j=0; j<nz; ++j){
        wavefield[i*nx+j] = newfield[i*nx+j];
      }
    }
    
    if( it % 100 ==0){
      avgRelDiff.push_back(avgRelDiffFroNorm/it);
      std::cout << "Iteration " << it;
      std::cout << ", avg. relative diff. Fro. norm: " << avgRelDiffFroNorm/it << std::endl;
    }

  }


  delete[] vel;
  delete[] oldfieldC;
  delete[] wavefieldC;
  delete[] newfieldC;
 }
  
  // write avgRelDiff to a file
  std::ofstream diffFile;
  if(typeid(dummyvar) == typeid(double)){
    diffFile.open("Inter/layerDiffDbl.txt");
  } else if(typeid(dummyvar) == typeid(float)){
    diffFile.open("Inter/layerDiffFlt.txt");
  }
  for(int i=0; i<avgRelDiff.size(); ++i){
    diffFile << avgRelDiff[i] << std::endl;
  }
  diffFile.close();

  return 0;
  
}
